/*
* Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, see <http://www.gnu.org/licenses/&gt;.
*
*/
#include "noticeAlarm.h"
#include "ui_noticeAlarm.h"
#include <QPalette>
#include <QSqlTableModel>
#include <QMediaPlayer>
#include <QMediaPlaylist>
#include <QDebug>
#include <QPainterPath>
#include <X11/Xlib.h>
#include "clockentitydao.h"
#include "xatom-helper.h"
#include "mediaplayerpool.h"
#include "gsettingsubject.h"
#include <math.h>
#include <QThread>
#include <QBitmap>
#include "theme.h"

extern void qt_blurImage(QImage &blurImage, qreal radius, bool quality, int transposed);
/**
 * @brief
 * @param
 * @param      num 闹钟序号 倒计时传-1
 *
 * @return 返回说明
 */
Natice_alarm::Natice_alarm(int close_time, int num, QWidget *parent , QString clockId) :
    QWidget(parent),
    ui(new Ui::Natice_alarm),
    timer_value(close_time),
    num_flag(num),
    m_clockId(clockId)
{
    ui->setupUi(this);
    m_musicPlayer = new MMediaPlayer(this);
    m_playList = new MMediaPlaylist(m_musicPlayer);
    m_playList->setPlaybackMode(MMediaPlaylist::CurrentItemInLoop);
    m_selectBtnUtil = new SelectBtnUtil();
    //响铃提示
    this->setWindowTitle(tr("Ring prompt"));
    this->setWindowIcon(QIcon::fromTheme("kylin-alarm-clock"));
    //多少秒后自动关闭
    ui->autoCloseTime->setAlignment(Qt::AlignRight);
    QPalette pa;
    pa.setColor(QPalette::WindowText,Qt::gray);
    ui->autoCloseTime->setPalette(pa);

    //this->setProperty("blurRegion", QRegion(QRect(1, 1, 1, 1)));
    //保存一个多少秒后关闭，用于稍后提醒的显式
    timer_value2 = timer_value;
    //右上角关闭
    connect(ui->closeTitleBtn, SIGNAL(clicked()), this, SLOT(set_dialog_close()) );
    //稍后提醒
    connect(ui->remindLateBtn, SIGNAL(clicked()), this, SLOT(show_again()) );
    //进行 剩余秒数显示 的定时
    timer = new QTimer();
    //剩余秒 减1
    connect(timer, SIGNAL(timeout()), this, SLOT(close_music()));
    timer->setInterval(1000);

    timer_xumhuan = new QTimer();
    //稍后提醒的定时
    connect(timer_xumhuan, SIGNAL(timeout()), this, SLOT(ring()));
    timer_xumhuan->setInterval(1000);
    model_clock = clock_sql::getClockTable(this);
    natice_init();
    QPalette palette1 = ui->closeTitleBtn->palette();
    QColor ColorPlaceholderText1(255,255,255,0);
    QBrush brush;
    brush.setColor(ColorPlaceholderText1);
    ui->closeTitleBtn->setPalette(palette1);

    ui->closeTitleBtn->setIcon(QIcon::fromTheme("window-close-symbolic"));
    ui->closeTitleBtn->setProperty("isWindowButton", 0x2);
    ui->closeTitleBtn->setProperty("useIconHighlightEffect", 0x8);
    ui->closeTitleBtn->setFlat(true);
    //启用主题框架不需要代码
    ui->titleIcon->setPixmap(QIcon::fromTheme("kylin-alarm-clock").pixmap(24,24));
    //    主题框架1.0.6-5kylin2

    //配置重要按钮 关闭
    ui->remindLateBtn->setProperty("isImportant", true);
    ui->closeTitleBtn->setFocusPolicy(Qt::NoFocus);
    //数据库
    model_setup = clock_sql::getSetupTable(this);
    //倒计时初始化无稍后提醒
    if(num_flag < 0){
        hideRemindBtn();
    }
    settingsStyle();
    //初始化锁屏dbus客户端
    m_screenSaveIface = new QDBusInterface(SSWND_DBUS_SERVICE,
                                      SSWND_DBUS_PATH,
                                      SSWND_DBUS_INTERFACE);
    //初始化使用不锁屏
    this->setAttribute(Qt::WA_TranslucentBackground);//设置窗口背景透明
   installEventFilter(this);
//   this->setWindowFlags(Qt::FramelessWindowHint|Qt::ToolTip); //设置无边框窗口
//   this->setAttribute(Qt::WA_AlwaysShowToolTips);
   Qt::WindowFlags m_flags = windowFlags();
   this->setWindowFlags(m_flags | Qt::WindowStaysOnTopHint);
//   添加窗管协议
//   XAtomHelper::setStandardWindowHint(this->winId());
   kdk::UkuiStyleHelper::self()->removeHeader(this);
   m_primaryManager = new PrimaryManager();

}

void Natice_alarm::settingsStyle()
{

    GsettingSubject * subject = GsettingSubject::getInstance();
    connect(subject,&GsettingSubject::fontChanged, this,[=](int size){
        this->CURRENT_FONT_SIZE=size;
        this->updateFront(size);
    });
    connect(subject,&GsettingSubject::iconChnaged, this,[=](){
        ui->titleIcon->setPixmap(QIcon::fromTheme("kylin-alarm-clock").pixmap(24,24));
    });
    subject->iniFontSize();
    //锁屏绑定screenSaveStyle
    connect(subject,&GsettingSubject::screenSaveLock, this,[=](){
        this->m_screenSaveOn = true;
        screenSaveStyle();
    });
    connect(subject,&GsettingSubject::screenSaveUnlock, this,[=](){
        this->m_screenSaveOn = false;
        screenSaveStyle();
    });
    subject->iniScreenSaveState();

}

void Natice_alarm::updateFront(int size)
{
    double font = 1.15;
    int pointSize = round(font*size);
    updateLabelFront(ui->titleLabel,pointSize);
    updateLabelFront(ui->timeEndInfo,pointSize);
    updateLabelFront(ui->autoCloseTime,pointSize);
    QFont tempFont;
    tempFont.setPixelSize(pointSize);
    ui->remindLateBtn->setFont(tempFont);
    //稍后提醒按钮最大号字体省略
    QString remindbtninfo = tr("Remind later");
    if(size >= 14){
        ui->remindLateBtn->setToolTip(remindbtninfo);
        int limitsize = 8;
        if(remindbtninfo.length() > limitsize){
            remindbtninfo = remindbtninfo.left(limitsize);
            remindbtninfo = remindbtninfo+"...";
        }
    }
    ui->remindLateBtn->setText(remindbtninfo);
}
void Natice_alarm::updateLabelFront(QLabel *label, int size)
{
    QString styleSheet = "font-size:";
    styleSheet.append(QString::number(size)).append("px;");
    label->setStyleSheet(styleSheet);
}

Natice_alarm::~Natice_alarm()
{
    delete timer;
    delete timer_xumhuan;
    delete ui;
}
/**
 * @brief 初始化铃声
 * @param
 * @param
 *
 * @return 返回说明
 */
void Natice_alarm::natice_init()
{
    if(num_flag >= 0)
    {
        auto sqlQuery = ClockEntityDao::getClockByPK(m_clockId);
        //更新闹钟名称
        QString info = model_clock->index(num_flag, 0).data().toString();
        info = Utils::changeNumToStr(info.toInt());
        info = info.append(":");
        info = info.append(Utils::changeNumToStr(model_clock->index(num_flag, 1).data().toInt()));
        info = info.append(" ");
        QString clockName = model_clock->index(num_flag, 13).data().toString();
        int sizeLimit = 3;
        if(clockName.length()>sizeLimit){
            //名字超长
            ui->timeEndInfo->setToolTip(info+clockName);
            clockName = clockName.left(sizeLimit);
            clockName = clockName+"...";
        }
        info = info.append(clockName);
        ui->timeEndInfo->setText(info);
        //稍后提醒逻辑
        QString remindLate = model_clock->index(num_flag, 15).data().toString();
        if(remindLate=="none"||remindLate==tr("none")){
            hideRemindBtn();
        }else{
            showRemindBtn();
        }
    } else {
        //倒计时
        ui->timeEndInfo->setText(tr("Time out")+"！");
    }
}

void Natice_alarm::closeMusic()
{

}
//重新加载最新音乐
void Natice_alarm::refreshMusic()
{
    QSqlTableModel *model = new QSqlTableModel(this);
    model->setTable("clock");
    model->setEditStrategy(QSqlTableModel::OnManualSubmit);
    model->select();
    model_setup->select();
    QString bellId="";
    if(num_flag >= 0){
        auto sqlQuery = ClockEntityDao::getClockByPK(m_clockId);
        bellId = sqlQuery.value( 2).toString();
    } else {
        bellId = model_setup->index(0, 1).data().toString();
    }
    QString noneId = m_selectBtnUtil->getBellIdByNameEn("none");
    if(bellId!=noneId){
        QString path = m_selectBtnUtil->getBellPathById(bellId);
        m_playList->clear();
        m_playList->addMedia(QUrl::fromLocalFile(path));
        m_musicPlayer->setPlaylist(m_playList);
        int muteOn = model_setup->index(0, 0).data().toInt();
        if(muteOn){
            qDebug()<<"dbq-静音";
            m_musicPlayer->play();
            m_musicPlayer->pause();
        }else{
            m_musicPlayer->setVolume(  100 );
            m_musicPlayer->play();
        }
    }
}

void Natice_alarm::hideRemindBtn()
{
    ui->widget_3->hide();
    this->setFixedSize(288,90);

}

void Natice_alarm::showRemindBtn()
{
    ui->widget_3->show();
    this->setFixedSize(288,120);
}

void Natice_alarm::playMusic()
{
    timer_value =  timer_value2;
    m_musicPlayer->setVolume(100);
    timer->start();
    refreshMusic();
}
void Natice_alarm::showThisWindow()
{
    kdk::UkuiStyleHelper::self()->removeHeader(this);
    this->show();
    m_primaryManager->moveUnderMultiScreen(PrimaryManager::UP_RIGHT,this,1);
}
void Natice_alarm::screenSaveStyle()
{
    if(this->isVisible()){
        this->hide();
        qDebug()<<"dbq-m_screenSaveOn"<<m_screenSaveOn;
        if(m_screenSaveOn){
//            setWindowFlags(Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint | Qt::X11BypassWindowManagerHint);
            this->setWindowFlags(Qt::FramelessWindowHint|Qt::ToolTip); //设置无边框窗口
            this->setAttribute(Qt::WA_AlwaysShowToolTips);
            //圆角
            QBitmap bmp(this->size());
            bmp.fill();
            QPainter p(&bmp);
            p.setRenderHint(QPainter::Antialiasing);  // 反锯齿;
            p.setPen(Qt::NoPen);
            p.setBrush(palette().color(QPalette::Base));
            p.drawRoundedRect(bmp.rect(),WINDOWN_RADIUS,WINDOWN_RADIUS);
            setMask(bmp);

            QThread::sleep(1);
            m_screenSaveIface = new QDBusInterface(SSWND_DBUS_SERVICE,
                                              SSWND_DBUS_PATH,
                                              SSWND_DBUS_INTERFACE);
            if ( m_screenSaveIface&& m_screenSaveIface->isValid()) {
                QDBusReply<int> reply =  m_screenSaveIface->call("RegisteSubWnd", this->winId());
                if(reply.isValid()) {
                    qDebug() << "Result :"<<reply.value();
                } else {
                    qWarning() << "Result invalid！！";
                }
            }else{
                qDebug()<<"dbq-not lock";
            }
        }else{
            Qt::WindowFlags m_flags = windowFlags();
            this->setWindowFlags(m_flags | Qt::WindowStaysOnTopHint);
//             添加窗管协议
//            XAtomHelper::setStandardWindowHint(this->winId());
            kdk::UkuiStyleHelper::self()->removeHeader(this);
        }
        update();
        this->showThisWindow();
    }

}

bool Natice_alarm::eventFilter(QObject *obj, QEvent *e)
{
    switch (e->type()) {
    case QEvent::MouseButtonPress: {
        QMouseEvent *event = static_cast<QMouseEvent*>(e);

        if (event->button() == Qt::LeftButton) {
            QWidget *w = qobject_cast<QWidget *>(obj);
            if (w->cursor().shape() != Qt::ArrowCursor)
                return false;
            mousePressEvent(event);
            return false;
        }
        return false;
    }
    case QEvent::MouseMove: {
        QMouseEvent *event = static_cast<QMouseEvent*>(e);
        if (qAbs(event->pos().x() - dragPosition.x()) < 2 ||
            qAbs(event->pos().y() - dragPosition.y()) < 2) {
            return true;
        }
        //move request
        mouseMoveEvent( event);
        return false;
    }
    case QEvent::MouseButtonRelease: {
        QMouseEvent *event = static_cast<QMouseEvent*>(e);
        mouseReleaseEvent( event);
        return false;
    }
    case QEvent::Move: {
        return false;
    }
    default:
        return false;
    }
}
void Natice_alarm::mousePressEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) {
        this->dragPosition = event->globalPos() - frameGeometry().topLeft();
        this->mousePressed = true;
    }
    QWidget::mousePressEvent(event);
}

void Natice_alarm::mouseReleaseEvent(QMouseEvent *event)
{
    if (event->button() == Qt::LeftButton) {
        this->mousePressed = false;
    }
    this->setCursor(Qt::ArrowCursor);
    QWidget::mouseReleaseEvent(event);
}

void Natice_alarm::mouseMoveEvent(QMouseEvent *event)
{

    if (this->mousePressed) {
        move(event->globalPos() - this->dragPosition);
        this->setCursor(Qt::ClosedHandCursor);
    }
    QWidget::mouseMoveEvent(event);



}

//窗口关闭
/**
 * @brief 铃声、窗体、timer关闭
 * @param
 * @param
 *
 * @return 返回说明
 */
void Natice_alarm::set_dialog_close()
{
    timer->stop();
    m_musicPlayer->stop();
    if(num_flag >= 0)
    {
        this->close();
    }else{ //倒计时的隐藏不关闭
        this->hide();
    }
}
//关闭音乐
/**
 * @brief 剩余时间读秒 全关闭
 * @param
 * @param
 *
 * @return 返回说明
 */
void Natice_alarm::close_music()
{
    if (timer_value == 0) {
        set_dialog_close();
    }
    timer_value--;
    ui->autoCloseTime->setText(QString::number(timer_value)+tr(" Seconds to close"));
}
//绘制背景
// Draw background
void Natice_alarm::paintEvent(QPaintEvent *event)
{
    if(m_screenSaveOn){
        Q_UNUSED(event);
        QPainter p(this);
        p.setRenderHint(QPainter::Antialiasing);
        QPainterPath rectPath;
        int SHADOW_RANGE = 1;
        rectPath.addRoundedRect(this->rect().adjusted(SHADOW_RANGE, SHADOW_RANGE, -SHADOW_RANGE, -SHADOW_RANGE), WINDOWN_RADIUS, WINDOWN_RADIUS);
        // 画一个黑底

        QPixmap pixmap(this->rect().size());
        pixmap.fill(Qt::transparent);
        QPainter pixmapPainter(&pixmap);
        pixmapPainter.setRenderHint(QPainter::Antialiasing);
        auto shadowColor = palette().text().color();
        shadowColor.setAlphaF(SHADOWCOLOR_ALPHAF);
        pixmapPainter.setBrush(shadowColor);
        pixmapPainter.setPen(Qt::transparent);
        pixmapPainter.drawPath(rectPath);
        pixmapPainter.end();
        // 模糊这个黑底
        QImage img = pixmap.toImage();
        qt_blurImage(img, VAGUE_RADIUS, false, false);
        // 挖掉中心
        pixmap = QPixmap::fromImage(img);
        QPainter pixmapPainter2(&pixmap);
        pixmapPainter2.setRenderHint(QPainter::Antialiasing);
        pixmapPainter2.setCompositionMode(QPainter::CompositionMode_Clear);
        pixmapPainter2.setPen(Qt::transparent);
        pixmapPainter2.setBrush(Qt::transparent);
        pixmapPainter2.drawPath(rectPath);
        // 绘制阴影
        p.drawPixmap(this->rect(), pixmap, pixmap.rect());
        QStyleOption opt;
        opt.init(this);
        // 绘制一个背景
        p.save();

        //描边
        QColor borderColor = palette().text().color();
        borderColor.setAlphaF(BORDERCOLOR_ALPHAF);
        p.setPen(borderColor);
        p.translate(BORDER_RANGE, BORDER_RANGE);
        p.setBrush(palette().color(QPalette::Base));
        p.drawPath(rectPath);
        p.restore();
    }else{
        QPainter p(this);
        p.setRenderHint(QPainter::Antialiasing);  // 反锯齿;
        QPainterPath rectPath;
        rectPath.addRect(this->rect());
        p.fillPath(rectPath,palette().color(QPalette::Base));
        p.setBrush(palette().color(QPalette::Base));
        p.setPen(Qt::transparent);
        p.drawPath(rectPath);
    }

}
//再次弹出
//Eject again
void Natice_alarm::show_again()
{
    this->hide();
    //刷新数据
    model_setup->select();
    QString info = model_clock->index(num_flag, 15).data().toString();
    int remind = getRemindStatusByName(info);
    if (remind == 1) {
        ring_num = 300;
    } else if(remind == 2) {
        ring_num = 600;
    } else if(remind == 3) {
        ring_num = 1200;
    } else if(remind == 4) {
        ring_num = 1800;
    } else if(remind == 5) {
        ring_num = 3600;
    }
    timer_value =  timer_value2;
    timer_xumhuan->start();
    timer->stop();
    m_musicPlayer->stop();
}

int Natice_alarm::getRemindStatusByName(QString name)
{
    int status = 0;
    if(name.compare("none") == 0 || name.compare("不需要") == 0){
        status = 0;
    }else if(name.compare("five mins late") == 0 || name.compare("5分钟后") == 0){
        status = 1;
    }else if(name.compare("ten mins late") == 0 || name.compare("10分钟后") == 0){
        status = 2;
    }else if(name.compare("twenty mins late") == 0 || name.compare("20分钟后") == 0){
        status = 3;
    }else if(name.compare("thirsty mins late") == 0 || name.compare("30分钟后") == 0){
        status = 4;
    }else if(name.compare("one hour late") == 0 || name.compare("1小时后") == 0){
        status = 5;
    }
    return status;
}
//响铃
//Ring a bell 每一秒减一
void Natice_alarm::ring()
{
    ring_num--;
    if (ring_num == 0) {
        timer_xumhuan->stop();
        if(num_flag >= 0){
            bool result = ClockEntityDao::checkClockExist(m_clockId);
            if(result){
                natice_init();
                this->showThisWindow();
                screenSaveStyle();
                timer->start();
                refreshMusic();
            }else{
                qDebug()<<"dbq-闹钟被删-啥也不做";
            }
        }else{
            natice_init();
            this->showThisWindow();
            screenSaveStyle();
            timer->start();
            refreshMusic();
        }

    }
}





void Natice_alarm::closeEvent(QCloseEvent *event)
{
    timer->stop();
    m_musicPlayer->stop();
    if(num_flag >= 0)
    {
        event->accept();//接受
        this->close();
    }else{
        event->ignore();//忽视
        this->hide();
    }
}
